==> 0.4.0%2B1.tar.gz: android/src/main/kotlin/com/snowm/scanner/services/SnowMBackgroundScanningService.kt <==
package com.snowm.scanner.services

import android.app.NotificationChannel
import android.app.NotificationManager
import android.app.PendingIntent
import android.app.Service
import android.content.Context
import android.content.Intent
import android.os.Build
import android.os.IBinder
import android.util.Log
import androidx.annotation.RequiresApi
import androidx.core.app.NotificationCompat
import com.snowm.scanner.R
import com.snowm.scanner.SnowmScannerPlugin
import com.snowm.scanner.helpers.BeaconScanner
import com.snowm.scanner.helpers.BeaconScanner.OnBeaconScanned
import com.snowm.scanner.helpers.MqttHelper
import com.snowm.scanner.helpers.NotificationHelper.removeNotification
import com.snowm.scanner.helpers.NotificationHelper.sendNotification
import com.snowm.scanner.helpers.PermissionManager
import com.snowm.scanner.models.SnowMiBeacon
import org.altbeacon.beacon.Region
import org.altbeacon.beacon.powersave.BackgroundPowerSaver
import java.util.*
import kotlin.collections.ArrayList

open class SnowMBackgroundScanningService : Service(), PermissionManager.BluetoothStateListener {
    lateinit var customData: Map<String, Any>
    private lateinit var uuids: ArrayList<String>
    private lateinit var mqttHelper: MqttHelper
    private lateinit var beaconScanner: BeaconScanner
    private lateinit var permissionManager: PermissionManager

    private lateinit var regionMap: HashMap<String, Boolean>

    override fun onBind(intent: Intent): IBinder? {
        return null
    }

    companion object {
        fun initializeWithApplication(context: Context) {
            BackgroundPowerSaver(context)
        }
    }

    private var onBeaconScanned: OnBeaconScanned = object : OnBeaconScanned {
        override fun onBeaconRanged(beacons: ArrayList<SnowMiBeacon>) {
            val validBeacons: ArrayList<SnowMiBeacon> = ArrayList()
            beacons.forEach() {
                mqttHelper.sendIBeaconInformation(it)
                if (uuids.contains(it.uuid))
                    validBeacons.add(it)
            }
            if (validBeacons.count() > 0)
                onBeaconsDetected(validBeacons)
        }

        override fun didEnterRegion(p0: Region?) {
            val uuid = p0?.id1.toString()
            if (!uuids.contains(uuid)) return
            if (!regionMap.values.contains(true)) onRegionEntered()

            regionMap[uuid] = true
        }

        override fun didExitRegion(p0: Region?) {
            val uuid = p0?.id1.toString()
            if (!uuids.contains(uuid)) return
            regionMap[uuid] = false
            if (!regionMap.values.contains(true)) onRegionExited()

        }
    }

    @RequiresApi(api = Build.VERSION_CODES.O)
    override fun onCreate() {
        super.onCreate()
        Log.d("BackgroundService", "created")
        mqttHelper = MqttHelper(this)
        beaconScanner = BeaconScanner(this)
        permissionManager = PermissionManager(this)
        beaconScanner.monitorBeacons("background")
        beaconScanner.setOnBeaconScanner("background", onBeaconScanned)
        permissionManager.bluetoothState("background", this)
    }

    override fun onStartCommand(intent: Intent, flags: Int, startId: Int): Int {
        val extra = intent.extras ?: return START_NOT_STICKY
        val title = extra.getString("title")
        val body = extra.getString("body")
        uuids = extra.getStringArrayList("uuids")
        uuids.forEach {
            regionMap[it] = false
        }
        customData = extra.getSerializable("customData") as Map<String, Any>
        createNotificationChannel(this)
        val activityIntent = Intent()
        activityIntent.setPackage(this.packageName)
        activityIntent.action = "android.intent.action.MAIN"
        activityIntent.flags = Intent.FLAG_ACTIVITY_SINGLE_TOP
        val contentIntent = PendingIntent.getActivity(this, 0, activityIntent, 0)
        val notification = NotificationCompat.Builder(this, "snowm_scanner").setContentTitle(title).setContentText(body).setSmallIcon(R.drawable.skel_logo).setContentIntent(contentIntent).build()
        startForeground(1, notification)
        return START_STICKY
    }

    private fun createNotificationChannel(context: Context) {
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
            val importance = NotificationManager.IMPORTANCE_DEFAULT
            val channel = NotificationChannel("snowm_scanner", "Background Scanner", importance)
            channel.description = "Notification alerts for beacon scanner."
            val notificationManager = context.getSystemService(NotificationManager::class.java)
            notificationManager.createNotificationChannel(channel)
        }
    }

    override fun onDestroy() {
        super.onDestroy()
        stopScan()
    }

    open fun onBeaconsDetected(beacons: ArrayList<SnowMiBeacon>) {}

    open fun onRegionEntered() {
        beaconScanner.rangeBeacons("background")
    }

    open fun onRegionExited() {
        beaconScanner.stopRanging("background")
    }

    private fun stopScan() {
        beaconScanner.stopRanging("background")
        beaconScanner.stopMonitoring("background")
    }

    override fun onStateChanged(state: PermissionManager.BluetoothState) {
        when (state) {
            PermissionManager.BluetoothState.OFF -> {
                sendNotification(20, "Bluetooth State", this, applicationInfo.nonLocalizedLabel.toString(), "Please turn on the bluetooth to track your attendance record.", true)
            }
            PermissionManager.BluetoothState.ON -> {
                removeNotification(this, 20)
            }
            else -> removeNotification(this, 20)
        }
    }
}

==> 0.7.0%2B1.tar.gz: android/src/main/kotlin/com/snowm/scanner/services/SnowMBackgroundScanningService.kt <==
package com.snowm.scanner.services

import android.app.NotificationChannel
import android.app.NotificationManager
import android.app.PendingIntent
import android.app.Service
import android.content.Context
import android.content.Intent
import android.os.Build
import android.os.IBinder
import android.util.Log
import androidx.annotation.RequiresApi
import androidx.core.app.NotificationCompat
import com.snowm.scanner.R
import com.snowm.scanner.SnowmScannerPlugin
import com.snowm.scanner.helpers.BeaconScanner
import com.snowm.scanner.helpers.BeaconScanner.OnBeaconScanned
import com.snowm.scanner.helpers.MqttHelper
import com.snowm.scanner.helpers.NotificationHelper.removeNotification
import com.snowm.scanner.helpers.NotificationHelper.sendNotification
import com.snowm.scanner.helpers.PermissionManager
import com.snowm.scanner.models.SnowMiBeacon
import org.altbeacon.beacon.Region
import org.altbeacon.beacon.powersave.BackgroundPowerSaver
import java.sql.Struct
import java.util.*
import kotlin.collections.ArrayList
import kotlin.collections.HashMap
import kotlin.properties.Delegates

open class SnowMBackgroundScanningService : Service(), PermissionManager.BluetoothStateListener {
    lateinit var customData: Map<String, Any>
    private lateinit var uuids: ArrayList<String>
    private lateinit var mqttHelper: MqttHelper
    private lateinit var beaconScanner: BeaconScanner
    private lateinit var permissionManager: PermissionManager
    var backgroundBetweenScanPeriod by Delegates.notNull<Long>()
    var backgroundScanPeriod by Delegates.notNull<Long>()
    private var regionMap: HashMap<String, Boolean> = HashMap()

    override fun onBind(intent: Intent): IBinder? {
        return null
    }

    companion object {
        var scanId: String? = null
        fun initializeWithApplication(context: Context) {
//            BackgroundPowerSaver(context)
        }
    }

    private var onBeaconScanned: OnBeaconScanned = object : OnBeaconScanned {
        override fun onBeaconRanged(beacons: ArrayList<SnowMiBeacon>) {
            val validBeacons: ArrayList<SnowMiBeacon> = ArrayList()
            beacons.forEach() {
                mqttHelper.sendIBeaconInformation(it)
                if (uuids.contains(it.uuid))
                    validBeacons.add(it)
            }
            if (validBeacons.count() > 0) {
                onBeaconsDetected(validBeacons)
            }
        }

        override fun didEnterRegion(p0: Region?) {
            val uuid = p0?.id1.toString()
            if (!uuids.contains(uuid)) return
            if (!regionMap.values.contains(true)) onRegionEntered()

            regionMap[uuid] = true
        }

        override fun didExitRegion(p0: Region?) {
            val uuid = p0?.id1.toString()
            if (!uuids.contains(uuid)) return
            regionMap[uuid] = false
            if (!regionMap.values.contains(true)) onRegionExited()

        }
    }

    override fun onCreate() {
        super.onCreate()
        Log.d("BackgroundService", "created")
        mqttHelper = MqttHelper(this)
        beaconScanner = BeaconScanner(this)
        permissionManager = PermissionManager(this)
        beaconScanner.monitorBeacons("background")

        permissionManager.bluetoothState("background", this)
    }

    override fun onStartCommand(intent: Intent, flags: Int, startId: Int): Int {
        val extra = intent.extras ?: return START_NOT_STICKY
        val title = extra.getString("title")
        val body = extra.getString("body")
        scanId = extra.getString("scanId")

        uuids = extra.getStringArrayList("uuids")
        uuids.forEach {
            regionMap[it] = false
        }
        customData = extra.getSerializable("customData") as Map<String, Any>
        var backgroundBetweenScanPeriod: Long = extra.getLong("backgroundBetweenScanPeriod")
        var backgroundScanPeriod: Long = extra.getLong("backgroundBetweenScanPeriod")
        beaconScanner.rangeBeacons("background", backgroundBetweenScanPeriod, backgroundScanPeriod)
        beaconScanner.setOnBeaconScanner("background", onBeaconScanned)
        onScannerRegistered(scanId = scanId!!, customData = customData!!);
        createNotificationChannel(this)
        val activityIntent = Intent()
        activityIntent.setPackage(this.packageName)
        activityIntent.action = "android.intent.action.MAIN"
        activityIntent.flags = Intent.FLAG_ACTIVITY_SINGLE_TOP
        val contentIntent = PendingIntent.getActivity(this, 0, activityIntent, 0)
        val notification = NotificationCompat.Builder(this, "snowm_scanner").setContentTitle(title).setContentText(body).setSmallIcon(R.drawable.skel_logo).setContentIntent(contentIntent).build()
        startForeground(1, notification)
        return START_STICKY
    }

    private fun createNotificationChannel(context: Context) {
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
            val importance = NotificationManager.IMPORTANCE_DEFAULT
            val channel = NotificationChannel("snowm_scanner", "Background Scanner", importance)
            channel.description = "Notification alerts for beacon scanner."
            val notificationManager = context.getSystemService(NotificationManager::class.java)
            notificationManager.createNotificationChannel(channel)
        }
    }

    override fun onDestroy() {
        super.onDestroy()
        scanId = null
        stopScan()
    }

    open fun onBeaconsDetected(beacons: ArrayList<SnowMiBeacon>) {}

    open fun onRegionEntered() {
        beaconScanner.rangeBeacons("background", backgroundBetweenScanPeriod, backgroundScanPeriod)
    }

    open fun onRegionExited() {
        beaconScanner.stopRanging("background")
    }

    open fun onScannerRegistered(scanId: String, customData: Map<String, Any>) {

    }

    fun stopScan() {
        beaconScanner.stopRanging("background")
        beaconScanner.stopMonitoring("background")
    }

    override fun onStateChanged(state: PermissionManager.BluetoothState) {
        when (state) {
            PermissionManager.BluetoothState.OFF -> {
                sendNotification(20, "Bluetooth State", this, applicationInfo.nonLocalizedLabel.toString(), "Please turn on the bluetooth to track your attendance record.", true)
            }
            PermissionManager.BluetoothState.ON -> {
                removeNotification(this, 20)
            }
            else -> removeNotification(this, 20)
        }
    }
}
